/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  drv82xx-RegMap.h - Declaration file for utility functions and global variables
 *  DRV8213_82x4EVM_FIRMWARE
 *  03/15/2023
 *  Pablo Armet
 *
 ******************************************************************************/
#ifndef DRV82xx_REGMAP_H_
#define DRV82xx_REGMAP_H_

#include "dataTypeDefinition.h"
#include "stdint.h"
#include "msp430.h"

// MACROs

// DRV8xx2 I2C Register Addresses
#define I2C_FAULT_STATUS            (uint8_t)(0x00)         // FAULT Register
#define I2C_RC_STATUS_1             (uint8_t)(0x01)         // RC_STATUS_1 Register
#define I2C_RC_STATUS_2             (uint8_t)(0x02)         // RC_STATUS_2 Register
#define I2C_RC_STATUS_3             (uint8_t)(0x03)         // RC_STATUS_3 Register
#define I2C_REG_STATUS_1            (uint8_t)(0x04)         // REG_STATUS_1 Register
#define I2C_REG_STATUS_2            (uint8_t)(0x05)         // REG_STATUS_2 Register
#define I2C_REG_STATUS_3            (uint8_t)(0x06)         // REG_STATUS_3 Register
#define I2C_CONFIG0                 (uint8_t)(0x09)         // CONFIG0 Register
#define I2C_CONFIG1                 (uint8_t)(0x0A)         // CONFIG1 Register
#define I2C_CONFIG2                 (uint8_t)(0x0B)         // CONFIG2 Register
#define I2C_CONFIG3                 (uint8_t)(0x0C)         // CONFIG3 Register
#define I2C_CONFIG4                 (uint8_t)(0x0D)         // CONFIG4 Register
#define I2C_REG_CTRL0               (uint8_t)(0x0E)         // REG CTRL0 Register
#define I2C_REG_CTRL1               (uint8_t)(0x0F)         // REG CTRL1 Register
#define I2C_REG_CTRL2               (uint8_t)(0x10)         // REG CTRL1 Register
#define I2C_RC_CTRL0                (uint8_t)(0x11)         // RC CTRL0 Register
#define I2C_RC_CTRL1                (uint8_t)(0x12)         // RC CTRL1 Register
#define I2C_RC_CTRL2                (uint8_t)(0x13)         // RC CTRL2 Register
#define I2C_RC_CTRL3                (uint8_t)(0x14)         // RC CTRL3 Register
#define I2C_RC_CTRL4                (uint8_t)(0x15)         // RC CTRL4 Register
#define I2C_RC_CTRL5                (uint8_t)(0x16)         // RC CTRL5 Register
#define I2C_RC_CTRL6                (uint8_t)(0x17)         // RC CTRL6 Register


// Bit Mask definitions for Registers

// Register 0x00: FAULT Register
#define OL_MASK               (0x01)         // Indicates open load condition
#define OT_MASK               (0x02)         // Logic OR of OTW and OTSD
#define STL_MASK              (0x04)         // Indicates motor stall
#define OCP_MASK              (0x08)         // Indicates over current fault condition
#define CPUV_MASK             (0x10)         // Indicates charge pump under voltage fault condition
#define UVLO_MASK             (0x20)         // Indicates an supply under voltage lockout fault condition
#define SPI_ERROR_MASK        (0x40)         // Indication SPI communication error
#define FAULT_MASK            (0x80)         // Inverse value of the nFAULT pin

// Structure variables for the DRV8xx2 registers
typedef struct DRV82xx_REG
{
    uint8_t FAULT_STATUS;
    uint8_t RC_STATUS_1;
    uint8_t RC_STATUS_2;
    uint8_t RC_STATUS_3;
    uint8_t REG_STATUS_1;
    uint8_t REG_STATUS_2;
    uint8_t REG_STATUS_3;
    uint8_t CONFIG0;
    uint8_t CONFIG1;
    uint8_t CONFIG2;
    uint8_t CONFIG3;
    uint8_t CONFIG4;
    uint8_t REG_CTRL0;
    uint8_t REG_CTRL1;
    uint8_t REG_CTRL2;
    uint8_t RC_CTRL0;
    uint8_t RC_CTRL1;
    uint8_t RC_CTRL2;
    uint8_t RC_CTRL3;
    uint8_t RC_CTRL4;
    uint8_t RC_CTRL5;
    uint8_t RC_CTRL6;
} DRV82xx_REG_t;                            // Starting address 0x6200 total 22 bytes

#endif // DRV82xx_REGMAP_H_
