/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  drv82xx.h - Declaration file for utility functions and global variables
 *  DRV8213_82x4EVM_FIRMWARE
 *  03/15/2023
 *  Pablo Armet
 *
 ******************************************************************************/
#ifndef DRV82xx_H_
#define DRV82xx_H_
/******************************************************************************
 * HEADER FILES
 *****************************************************************************/
#include "drv82xx/include/drv82xx-RegMap.h"

/******************************************************************************
 * VARIABLES
 *****************************************************************************/
volatile bool       ReadRegs;

/******************************************************************************
 * MACROS
 *****************************************************************************/
// GPIO Port 1 Definitions
#define PUSH               BIT2    // P1.2: [I] PUSH button function (not implemented in firmware)

// GPIO Port 2 Definitions
#define nFAULT             BIT0    // P2.0: [I] DRV nFAULT indicator (0: Fault detected; 1: No fault detection)
#define GAINSEL_pin        BIT1    // P2.1: [O] GAIN selection (0: 225µA/A; hi-z: 1125µA/A; 1: 5560µA/A)
#define nSTALL_pin         BIT3    // P2.3: [I] Stall indicator 

#define STALL              BIT3    // P2.3: [O] DRV82x4 stall indicator
#define RC_OUT             BIT1    // P2.1: [I] DRV82x4 RC_OUT ADC

// GPIO Port 3 Definitions
#define TP6                BIT0    // P3.0: [X] Test point 6
#define MCU_STATUS         BIT1    // P3.1: [O] MCU active status indicator
#define GUI_STATUS         BIT2    // P3.2: [O] GUI connection status indicator
#define nSLEEP             BIT7    // P3.7: [O] Device nSLEEP

//#define ENABLE             BIT3    // P3.3: [O] DRV enable indicator

// GPIO Port 4 Definitions
#define IN1_PIN            BIT1    // P4.1: IN1 PWM output to driver IN1
#define IN2_PIN            BIT3    // P4.3: IN2 PWM output to driver IN2

// GPIO Port 5 Definitions
#define IDR_1              BIT3    // P5.3: EVM ID1 setting
#define IDR_2              BIT4    // P5.4: EVM ID2 setting
#define IDR_3              BIT5    // P5.5: EVM ID3 setting
#define TP4                BIT6    // P5.6: Test Point TP4
#define TP5                BIT7    // P5.7: Test Point TP5

// GPIO Port 6 Definitions
#define VSEN_VM            BIT0    // P6.4: A0 - VM V-sense analog input
#define IPROPI             BIT1    // P6.5: A1 - IPROPI voltage
#define VREF_pin           BIT7    // P6.7: DAC1 - VREF analog output

#define RC_OUT_DAC         BIT6    // P6.6: [DAC] DAC0 for RC (ripple count) analogue output 

// GPIO Port 7 Definitions
  //NONE

// GPIO Port 8 Definitions
#define UART_TXD           BIT2    // P8.2: UART TXD
#define UART_RXD           BIT3    // P8.3: UART RXD
#define SMODE_pin          BIT5    // P8.5: SMODE selection
#define IMODE_pin          BIT6    // P8.6: IMODE selection

#define SDA                BIT5    // P8.5: DRV82x4 SDA 
#define SCL                BIT6    // P8.6: DRV82x4 SCL

// GPIO Port 9 Definitions
  //NONE

// SMODE GPIO macro
#define set_SMODE_hi              P8DIR |= SMODE_pin; P8OUT |= SMODE_pin
#define set_SMODE_lo              P8DIR |= SMODE_pin; P8OUT &= ~SMODE_pin
#define set_SMODE_hiz             P8DIR &= ~SMODE_pin

// IMODE GPIO macro
#define set_IMODE_hi              P8DIR |= IMODE_pin; P8OUT |= IMODE_pin
#define set_IMODE_lo              P8DIR |= IMODE_pin; P8OUT &= ~IMODE_pin
#define set_IMODE_hiz             P8DIR &= ~IMODE_pin

// GAINSEL GPIO macro
#define set_GAINSEL_225           P2DIR |= GAINSEL_pin; P2OUT &= ~GAINSEL_pin
#define set_GAINSEL_1125          P2DIR &= ~GAINSEL_pin
#define set_GAINSEL_5560          P2DIR |= GAINSEL_pin; P2OUT |= GAINSEL_pin

// nSLEEP GPIO macro
#define set_nSLEEP_hi             P3OUT |= nSLEEP
#define set_nSLEEP_lo             P3OUT &= ~nSLEEP

// IN1/IN2 GPIO macro
#define set_in1_hi                P4OUT |= IN1_PIN; P4SEL &= ~IN1_PIN
#define set_in1_lo                P4OUT &= ~IN1_PIN; P4SEL &= ~IN1_PIN
#define set_in2_hi                P4OUT |= IN2_PIN; P4SEL &= ~IN2_PIN
#define set_in2_lo                P4OUT &= ~IN2_PIN; P4SEL &= ~IN2_PIN

// Stall LED macro
#define set_stall_led_high        P2OUT |= STALL 
#define set_stall_led_low         P2OUT &= ~STALL

// PWM GPIO macro
#define read_nSLEEP_pin           P3IN & nSLEEP
#define read_nFAULT_pin           P2IN & nFAULT
#define read_SMODE_pin            P8IN & SMODE_pin
#define read_IMODE_pin            P8IN & IMODE_pin
#define toggle_status_led         P3OUT ^= MCU_STATUS

#define read_ID1_2_pin            P5IN & (IDR_1 | IDR_2)
#define read_ID3_pin              P5IN & (IDR_3)

#define read_PUSH_pin             P1IN & PUSH

// Stall detection
#define read_nSTALL_pin           P2IN & nSTALL_pin

// Set Step Mode
#define _PWM_TIMER_PERIOD 100
#define PWM_TIMER_PERIOD (_PWM_TIMER_PERIOD-1)
#define PWM_SCALE (_PWM_TIMER_PERIOD/100)
#define PWM_FREQ_200KHZ           0
#define PWM_FREQ_100KHZ           ID0
#define PWM_FREQ_50KHZ            ID1
#define PWM_FREQ_25KHZ            ID1 | ID0

#define TA0CCR1_DEFAULT           2
#define NumSamples                48
#define ADC12scale                68

#define REV_PG1_0                 0
#define REV_PG2_0                 1

#define RC_SCALE                  10     //Ripple count scale for DAC conversion

//Mask definitions
#define STALL_MASK                0x20  
#define I2CBC_MASK                0x04
#define IN1_MASK                  0x02  
#define IN2_MASK                  0x01
#define RC_CNT_MASK               0x01
#define RC_HIZ_MASK               0x20
#define CLR_CNT_MASK              0x04
#define EN_OUT_MASK               0x80
#define PMODE_MASK                0b00001000


#define LED_TIMEOUT 125                     // Count to blink LED with 1s period
#define REG_READ    62                      // 500ms interval to read and refresh register values to the GUI
#define RC_DAC      10                      //
#define DIR_CHNG    125                     //

/******************************************************************************
* DATA STRUCTURES
 *****************************************************************************/

// Ramp duty controller
typedef struct DRV_RMPCNTL
{
  volatile uint8_t rampDelayMax;   // Parameter: Maximum delay rate (0-25)
  volatile uint8_t rampDelayCount; // Variable: Incremental delay (0-25)
  volatile uint8_t setpointValue;  // Output: Target output(0-100)

} RMPCNTL_t;

typedef struct DRV82xx_Device_Obj
{
  float firmwareVersion;              // Firmware version

  /* Device Parameters*/
  volatile uint8_t nFault;            // 0: No fault , 1: Fault Fault indicator
  volatile uint8_t IMODE;             // binding IMODE
  volatile uint8_t nSTALL_Config;     // binding nSTALL Config   
  volatile uint8_t ClrFlt;            // binding Clear button
  volatile uint8_t GAINSEL;           // binding GAINSEL widget
  volatile uint8_t pwmFreq;           // binding PWM Frequency widget
  volatile uint8_t Init_STALL;        // binding Initiate Stall widget
  volatile uint8_t nSTALL;            //
  volatile uint8_t motor_poles;       // motor numbers of poles
  volatile uint8_t AutoRead;          // enable or disable GUI Auto Read
  float motor_resistance;             // user input motor resistance
  float motor_constant;               // user input motor constant

 // VM supply voltage
  volatile uint16_t VMvoltage;         // binding Vmvoltage  widget same as DRV8462"
  volatile uint16_t deviceID; 
  volatile uint16_t ipropiAdc;

  /*Ramp controller*/
  RMPCNTL_t rampInput1;
  RMPCNTL_t rampInput2;

  /*Control Reference*/
  volatile uint8_t in1;
  volatile uint8_t in2;

  /* Register Read Wite */
  volatile uint16_t   ManWriteAddr;
  volatile uint16_t   ManReadAddr;
  volatile uint16_t   ManWriteData;
  volatile uint16_t   ManReadData;
  volatile bool       ManWriteCmd;
  volatile bool       ManReadCmd;
  volatile bool       ReadCmd;

  volatile uint16_t   revID;

  volatile uint8_t StartDrive1;
  volatile uint8_t SMODE;           
  volatile float VREF;
  volatile uint8_t nSleep;   
  volatile uint8_t Stall_LED;       // Stall LED indicator
  volatile uint16_t RC_OUT_TP;      // Analog output from RC_OUT register value
  volatile uint8_t CtrlMode;        // 0: Load Control; 1: ripple counting
  volatile uint8_t dir;          // change direction when RC_CNT reaches threshold
  volatile uint8_t rc_freq;        // RC_OUT frequency used for KMC and KMC_SCALE auto calculations in the GUI
  volatile bool speed_reg;        // speed regulation enable/disable
  volatile bool voltage_reg;      // voltage regulation enable/disable indicator

  volatile uint8_t dir1;          // direction for Bridge 1
  volatile uint8_t pre_dir1;       // previous direction for Bridge 1
  
} DRV82xx_Device_Obj_t;             //Start Address = 0x006100; Length = 66 bytes

// Utility variables
volatile uint16_t Device_ID;
volatile unsigned char timerUpdate;
volatile unsigned char pwmState1;
volatile float VREF_previous;
volatile float RC_previous;
volatile unsigned int ipropi;
volatile int ADC12offset;
volatile unsigned char ADC12offsetpositive;
volatile unsigned char ADC12offsetnegative;
unsigned int LED_TOGGLE_CNT;                    // count to determine if LED should be toggled
unsigned int REG_UPDATE;                        // for periodic register update
unsigned int DIR_CHNG_UPDATE;
unsigned int clear_fault_loop;
volatile bool ignore_ramp;
volatile uint16_t RC_CNT_REG;
volatile uint8_t clr_fault;

/******************************************************************************
 * EXTERN DECLARATIONS
 *****************************************************************************/
extern volatile DRV82xx_Device_Obj_t gBrushed_Obj;
extern volatile DRV82xx_REG_t gDeviceI2CReg;
extern uint8_t DeviceGroup;

/******************************************************************************
 * API Prototypes
 *****************************************************************************/
void gpio_update(void);
void mcu_init(void);
void rc_out_dac_cal(void);
void rc_plot(void);
void getEVMID(void);
void drv82xx_init(void);
void brushed_init(void);
void pwmTimerConfig(void);
void adc12_Init(void);
void adcTimerConfig(void);
void UpdateipropiReading(void);
void GUIcurrentdisplay(void);
void UpdateDeviceControls(void);
void UpdateGUIControls(void);
void StartStopMotor(void);
void DRV_RampControl(void);
void drv_TimerUpdate(void);
void drv_PH_EN_RampControl(void);
void drv_PWM_RampControl(void);
void drv_PWM(void);
void drv_RampCTRL(volatile RMPCNTL_t* rmpCntl, uint8_t targetValue);
void gui_regUpdateTask(void);
void gui_regRead(void);
void ClearFaultPulse(void);
void ReadAllRegs(void);
void RC_CNT_plot(void);
void RC_CNT_DIR(void);
void ReverseDirection(void);

/*****************************************************************************/
#endif // DRV8xx2_H_
