/* --COPYRIG  HT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  drv82xx.c - Definition file for utility functions
 *  DRV8213_82x4EVM_FIRMWARE
 *  03/15/2023
 *  Pablo Armet
 *
 *******************************************************************************/

/********************************************************************************
 * FIRMWARE VERSION INFORMATION
 *******************************************************************************/
#define FIRMWARE_VERSION 0.29                 // Updates to support DRV8215 and 8235
//#define FIRMWARE_VERSION 0.28               // Updates for preliminary DRV8214EVM release
/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include "drv82xx/include/drv82xx.h"
#include "uart.h"
#include "I2C.h"
#include <stdio.h>

unsigned char loop;
unsigned long ipropi_sum = 0;
extern unsigned int ipropi_Raw[];
extern unsigned char SamplesCollected;
extern volatile unsigned char Adcresults;
unsigned char ta0ccr1_value = TA0CCR1_DEFAULT;
uint8_t RC_CNT_REG1 = 0;
uint8_t RC_CNT_REG2 = 0;
uint8_t CNT_DONE_temp = 0;
uint8_t evmID = 0;

    


/********************************************************************************
 * Data Structures exported for GUI Composer
 *******************************************************************************/
#pragma DATA_SECTION(gBrushed_Obj,".drvVarSection");
#pragma DATA_SECTION(gDeviceI2CReg,".drvRegSection");

volatile DRV82xx_Device_Obj_t gBrushed_Obj;
volatile DRV82xx_REG_t gDeviceI2CReg;

/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/
// Clock Initialization
static inline void clock_Init(void)
{
    PMMCTL0 = PMMPW | PMMCOREV_3;             // PMM Core Voltage 1.85 V for 20 MHz CPU operation
    __delay_cycles(100000);                   // Settling delay for the core regulator

    UCSCTL3 = SELREF_2;                       // Set DCO FLL reference = REFO
    UCSCTL4 |= SELA_2 + SELS_3;               // Set ACLK = REFO and SMCLK = DCO
    UCSCTL1 = DCORSEL_6;                      // Select DCO range 20MHz operation
    UCSCTL2 = 609;                            // FLL multiplier
    __delay_cycles(625000);                   // Settling delay for DCO, 32 x 32 x 20 MHz / 32768 Hz = 625000 = MCLK cycles
}

// GPIO Initialization. This block configures all the DRV8434S PINs.
    static inline void gpio_Init(void)
{
    // Port P1 configuration
    P1OUT = 0;                                                       // All outputs low
    P1DIR = 0;                                                       // Default inputs   

    // Port P3 configuration
    P3OUT = 0;                                                       // All outputs low
    P3DIR = 0;                                                       // Default inputs
    P3DIR = MCU_STATUS | GUI_STATUS | nSLEEP | TP6;                  // All outputs
    P3SEL = TP6;                                                     // TP6 = SMCLK

    //Port P4 configurations
    P4OUT = 0;
    P4DIR = 0;
    P4DIR = IN1_PIN | IN2_PIN;
    P4SEL = IN1_PIN | IN2_PIN;

    // Port P5 configuration
    P5OUT = 0;                                                       // All outputs low
    P5DIR = 0;                                                       // Default inputs
    P5DIR = TP4 | TP5;                                               // IDR_1, IDR_2, IDR_3, VREF+ and VREF- are inputs
    P5SEL = BIT1 | BIT0;                                             // P5.0 = VeREF+ and P5.1 = VeREF-

    // Port P7 configuration
    P7OUT = 0;                                                       // All outputs low
    P7DIR = 0;                                                       // Default inputs

    // Port P9 configuration
    P9OUT = 0;                                                       // All outputs low
    P9DIR = 0;                                                       // Default inputs
}

void gpio_update(void)
{
    if (gBrushed_Obj.deviceID == 8213)
    {
        // Port P2 configuration
        P2OUT = 0;                                                       // All outputs low
        P2DIR = 0;                                                       // Default inputs
        P2DIR = GAINSEL_pin;                                             // nFAULT and nSTALL inputs

        // Port P6 configuration
        P6OUT = 0;                                                       // All outputs low except M0_nSCS as high
        P6DIR = 0;                                                       // Default inputs
        P6SEL = VREF_pin | VSEN_VM;                                      // Analog inputs

        // Port P8 configuration
        P8OUT = 0;                                                       // All outputs low
        P8DIR = 0;                                                       // Default inputs
        P8DIR = SMODE_pin | IMODE_pin;                                   // All except MODE_CTL as inputs
        P8SEL = UART_RXD | UART_TXD;                                     // SPI and UART pins
    }
    else {
        // Port P2 configuration
        P2OUT = 0;                                                       // All outputs low
        P2DIR = 0;                                                       // Default inputs
        P2DIR = STALL;                                                   // nFAULT, nSTALL, and RC_OUT inputs

        // Port P6 configuration
        P6OUT = 0;                                                       // All outputs low except M0_nSCS as high
        P6DIR = 0;                                                       // Default inputs
        P6SEL = IPROPI | VSEN_VM | VREF_pin;                             // Analog inputs; RC_OUT and Vref_pin configured as DAC outputs

        // Port P8 configuration
        P8OUT = 0;                                                       // All outputs low
        P8DIR = 0;                                                       // Default inputs
        P8SEL = UART_RXD | UART_TXD | SDA | SCL;                         // I2C and UART pins configuration

        // DAC12 configuration for RC_OUT_DAC (analog voltage proportional to RC_CNT value)
        DAC12_0CTL0 = DAC12CALON | DAC12SREF_3 | DAC12IR | DAC12AMP_7;   // Configure DAC12_0 on P6.6 initiate DAC12_0 calibration
        while (DAC12_0CTL0 & DAC12CALON);                                // Wait for Calibration complete
        DAC12_0DAT = 0;                                                  // Default output 0V
        // End DAC12 configuration
    }

    // DAC12 configuration for VREF
    DAC12_1CTL0 = DAC12CALON | DAC12SREF_3 | DAC12IR | DAC12AMP_7;       // Configure DAC12_1 on P6.7 initiate DAC12_1 calibration
    while (DAC12_1CTL0 & DAC12CALON);                                    // Wait for Calibration complete
    DAC12_1DAT = 0;                                                      // Default output 0V
    // End DAC12 configuration

}

// Watchdog initialization
static inline void watchdog_Init(void)
{
    WDTCTL = WDT_ARST_250;                                           // Watchdog mode with 250ms reset time using ACLK
}

// Setup Timer_A2 for periodic wake-up from LPM0 and to Blink the STATUS LED
static inline void ta2_init(void)  //Will need to update to ta1
{
    // select SMCLK, divide by 8, stop
    TA2CTL = TASSEL_2 | ID_3 | MC_0 | TACLR;
    TA2CCR0 = 20001;                                                 // 8 ms interrupt SMCLK = 20 MHz, TACLK = 2.5 MHz
    TA2CCTL0 |= CCIE;                                                // TIMER_A3CCR0 dedicated interrupt
    TA2CTL |= MC_1;                                                  // Start Timer_A2 in up mode
}

// Timer Initialization for PWM outputs to driver
void pwmTimerConfig(void)
{
    // Initialize Timer B0 with 0% duty for IN1, IN2
    TB0CTL = TBSSEL_2 | MC_0 | TBCLR;                                // Timer clock is SMCLK 20MHz, timer halted and cleared
    TB0CCTL0 |= CCIE;                                                // Interrupt enable
    TB0CCR0 = PWM_TIMER_PERIOD;                                      // Timer period value

    TB0CCTL1 |= OUTMOD_7;                                            // Timer mode to PWM reset/set for IN1
    TB0CCTL3 |= OUTMOD_7;                                            // Timer mode to PWM reset/set for IN2

    TB0CCR1 = 0;                                                     // IN1 compare register reset
    TB0CCR3 = 0;                                                     // IN2 compare register reset

    TB0CTL = TBSSEL_2 | PWM_FREQ_25KHZ| MC_1 | TBCLR;                // Timer clock is SMCLK 20MHz, up mode, cleared to start counting from zero
    P4SEL |= IN1_PIN + IN2_PIN;                                      // IN1 and IN2 setup as timer PWM outputs
}

// Timer A0 Initialization for ADC12_A internal Timer trigger ADC12SHSx = 1
void adcTimerConfig(void)
{
    // Initialize Timer A0.1 with 0% duty ADC12_A internal Timer trigger ADC12SHSx  1
    TA0CTL = TASSEL_2 | PWM_FREQ_25KHZ | MC_0 | TACLR;               // Timer clock is SMCLK 20MHz, timer halted and cleared
    TA0CCR0 = PWM_TIMER_PERIOD;                                      // Timer period value
    TA0CCR1 = TA0CCR1_DEFAULT;                                       // Trigger for ADC12_A
    TA0CCTL1 |= OUTMOD_3;                                            // Timer mode to PWM reset/set
    TA0CTL |= MC_1 | TACLR;                                          // Start Timer_A0 in up mode
    P3SEL |= BIT0;                                                   // for probing ADC triggers with oscilloscope
}

// ADC12 Initialization to capture IPROPI values in the ADC12 ISR.
void adc12_Init(void)
{
    ADC12CTL0 = ADC12ON;                                             // Setup ADC12
    ADC12CTL1 = ADC12CONSEQ_3 + ADC12SHS_1;                          // Timer A0.1 triggers sampling ADC12ISSH +

    ADC12MCTL0 = ADC12INCH_1 + ADC12SREF_6;                          // A1 for IPROPI1 with VeRef+ reference voltage
    ADC12MCTL1 = ADC12EOS + ADC12INCH_1 + ADC12SREF_6;               // A1 for IPROPI1 with VeRef+ reference voltage. End of sequence.
    ADC12IE = ADC12IE1;                                              // Enable ADC Interrupt after sequence complete
    ADC12CTL0 |= ADC12ENC;                                           // Conversion enabled
}

// Read EVMID information of the EVM hardware
void getEVMID(void)
{
    /* Get identify of the EVM connected
     *
     * DRV8213/14/34 EVMs have on board ID resistors inputs on
     * the MCU GPIO. ID1 to ID3 inputs uniquely identify the EVM connected
     * as well as whether it has pre-production or production silicon.
     *
     *   IDx=1       IDx=0      Pin state
     *     H                    Pull-up installed
     *                  L       Pull-down installed
     *
     *   P5.3 = ID1
     *   P5.4 = ID2
     *   P5.5 = ID3
     *
     *           EVM HW       Package  Interface    Motor   ID1   ID2       EVM OPN
     * DRV8213   MD068-001      PWP     GPIO         BDC     H     H       DRV8213EVM
     * DRV8214   MD069-001      PWP     I2C          BDC     H     L       DRV8214EVM
     * DRV8234   MD069-002      PWP     I2C          BDC     L     L       DRV8234EVM
     * DRV8215   MD069-003      PWP     I2C          BDC     L     H       DRV8215EVM
     * DRV8235   MD069-004      PWP     I2C          BDC    Hi-Z   L       DRV8235EVM
     * 
     * 
     * ID3 = H, pre-production silicon and ID3 = L, production silicon
     */

    evmID = (read_ID1_2_pin) >> 3;
   // evmID_temp = (read_ID1_2_pin);                                 // Troubleshooting purpose
    // Follow Calypso process to drive ID1 high then low and see if it reads high-low or not to determine Hi-z

    bool highState = 0;
    bool lowState = 0;
    bool IN1_HiZ = 0;
    P5REN |= ( IDR_1 );         // Enable internal pull-up/pull-down resistor
    P5OUT |= ( IDR_1 );         // Internal pull-up, start with driving the pin high
    __delay_cycles(8000);
    highState = (P5IN & IDR_1); // Read the state of the ID 1 pin
    P5OUT &= ~( IDR_1 );        // Internal pull-down, drive the pin low
    __delay_cycles(8000);
    lowState = (P5IN & IDR_1);  // Read the state of the ID 1 pin
    P5REN &= ~( IDR_1 );        // Disable internal pull-up/pull-down resistor
    if (highState && !lowState) {IN1_HiZ = 1;} // If it followed the state of the internal resistors then it is Hi-Z. 
    
    if (IN1_HiZ)
    {
        gBrushed_Obj.deviceID = 8235;                                   // DRV8235 RTE
    }
    else
    {
        switch(evmID)
        {
            case 0:
                gBrushed_Obj.deviceID = 8234;   // DRV8234 RTE
                break;
            case 1:
                gBrushed_Obj.deviceID = 8214;   // DRV8214 RTE
                break;
            case 2:
                gBrushed_Obj.deviceID = 8215;   // DRV8215 RTE
                break;
            case 3:
                gBrushed_Obj.deviceID = 8213;   // DRV8213 RTE
                break;
            default:
                break;
        }
    }
    

    // ID3
    if (read_ID3_pin)                  // ID3 pull-up
        gBrushed_Obj.revID = REV_PG1_0;
    else                               // ID3 pull-down
        gBrushed_Obj.revID = REV_PG2_0;
}

/*******************************************************************************
 * API IMPLEMENTATION
 *******************************************************************************/
// Initialize brushed drivers GUI variables
void brushed_init()
{
    if (gBrushed_Obj.deviceID == 8213)
    {
        gBrushed_Obj.firmwareVersion = FIRMWARE_VERSION;
        gBrushed_Obj.nFault = 1;
        gBrushed_Obj.nSTALL = 1;
        gBrushed_Obj.IMODE = 0;
        gBrushed_Obj.SMODE = 0;
        gBrushed_Obj.nSTALL_Config = 1;
        gBrushed_Obj.Init_STALL = 0;
        gBrushed_Obj.GAINSEL = 0;
        gBrushed_Obj.pwmFreq = 4;
        gBrushed_Obj.StartDrive1 = 0;
        gBrushed_Obj.in1 = _PWM_TIMER_PERIOD;
        gBrushed_Obj.in2 = _PWM_TIMER_PERIOD;
        gBrushed_Obj.ipropiAdc = 0;
        gBrushed_Obj.VREF = 3.3;
        gBrushed_Obj.nSleep = 0;

        // Ramp controller init for in1
        gBrushed_Obj.rampInput1.rampDelayMax   = 10;
        gBrushed_Obj.rampInput1.rampDelayCount = 0;
        gBrushed_Obj.rampInput1.setpointValue  = _PWM_TIMER_PERIOD;

        // Ramp controller init for in2
        gBrushed_Obj.rampInput2.rampDelayMax = 10;
        gBrushed_Obj.rampInput2.rampDelayCount = 0;
        gBrushed_Obj.rampInput2.setpointValue = _PWM_TIMER_PERIOD;

        ignore_ramp = TRUE;         //ignore DRV_PWM() to avoid motor spinning during initialization        
    }
    else if (gBrushed_Obj.deviceID == 8214 || gBrushed_Obj.deviceID == 8234 || gBrushed_Obj.deviceID == 8215 || gBrushed_Obj.deviceID == 8235)
    {
        gBrushed_Obj.firmwareVersion = FIRMWARE_VERSION;
        gBrushed_Obj.nFault = 1;
        gBrushed_Obj.nSTALL = 1;
        gBrushed_Obj.IMODE = 0;
        gBrushed_Obj.SMODE = 0;
        gBrushed_Obj.nSTALL_Config = 1;
        gBrushed_Obj.Init_STALL = 0;
        gBrushed_Obj.GAINSEL = 0;
        gBrushed_Obj.pwmFreq = 4;
        gBrushed_Obj.StartDrive1 = 0;
        gBrushed_Obj.in1 = _PWM_TIMER_PERIOD;
        gBrushed_Obj.in2 = _PWM_TIMER_PERIOD;
        gBrushed_Obj.ipropiAdc = 0;
        gBrushed_Obj.VREF = 3.3;
        gBrushed_Obj.nSleep = 0;
        gBrushed_Obj.AutoRead = 0;
        gBrushed_Obj.ManReadAddr = 0;
        gBrushed_Obj.ManReadData = 0;

        // Ramp controller init for in1
        gBrushed_Obj.rampInput1.rampDelayMax   = 10;
        gBrushed_Obj.rampInput1.rampDelayCount = 0;
        gBrushed_Obj.rampInput1.setpointValue  = _PWM_TIMER_PERIOD;

        // Ramp controller init for in2
        gBrushed_Obj.rampInput2.rampDelayMax = 10;
        gBrushed_Obj.rampInput2.rampDelayCount = 0;
        gBrushed_Obj.rampInput2.setpointValue = _PWM_TIMER_PERIOD;

        ignore_ramp = TRUE;         //ignore DRV_PWM() to avoid motor spinning during initialization
        RC_CNT_REG = 0;
    }
    
}

// Read the GUI settings and update device control
void UpdateDeviceControls()
{

    // toggle MCU_STATUS LED if count is reached. every 1s
    if (LED_TOGGLE_CNT > LED_TIMEOUT)
    {
        toggle_status_led;
        LED_TOGGLE_CNT = 0;

        // Measure VM and update to GUI display
        ADC12CTL0 &= ~ADC12ENC;                                      // Disable ADC
        ADC12IE = 0;                                                 // Disable ADC interrupts
        ADC12CTL1 = ADC12SHP;                                        // configure for single conversion
        ADC12MCTL0 = ADC12INCH_0 + ADC12SREF_6;                      // A0 for VM measurement
        ADC12CTL0 |= ADC12SC + ADC12ENC;                             // Enable and start conversion
        while (ADC12CTL1 & ADC12BUSY);                               // Wait for conversion to complete
        gBrushed_Obj.VMvoltage = ADC12MEM0;                          // Conversion result stored in ADC12MEM0 sent to GUI display
        if (gBrushed_Obj.VMvoltage < ADC12scale)
            gBrushed_Obj.VMvoltage = 0;                              // VM is 0
        ADC12CTL0 &= ~ADC12ENC;                                      // Disable ADC
        Adcresults = 0;                                              // Reset array pointer
        adc12_Init();                                                // Reconfigure ADC for IPROPIx measurement
        // End measure VM and update to GUI display
    }

    // Read and refresh register values to the GUI every 250ms
    if (REG_UPDATE > REG_READ)
    {
        ReadRegs = 1;                                                // Enable read all Registers
        REG_UPDATE = 0;                                              // Reset register read interval timing
    } 

    if (gBrushed_Obj.nSleep && !(read_nSLEEP_pin))
    {
        set_nSLEEP_hi;                                               // Set nSLEEP HIGH
        set_in1_hi;                                                  // Set IN1 to GPIO HIGH
        set_in2_hi;                                                  // Set IN2 to GPIO HIGH
        gBrushed_Obj.in1 = 0;                                        // Set IN1 to 0 (100% Duty Cycle)
        gBrushed_Obj.in2 = 0;                                        // Set IN2 to 0 (100% Duty Cycle)
        gBrushed_Obj.rampInput1.setpointValue  = 0;                  // Set rampInput1.setpointValue = 0
        gBrushed_Obj.rampInput2.setpointValue  = 0;                  // Set rampInput2.setpointValue = 0
        ignore_ramp = TRUE;                                          // Ignore DRV_PWM()
        gBrushed_Obj.ManReadCmd = FALSE;
        gBrushed_Obj.ManWriteCmd = FALSE;
        _delay_cycles(30000);                                        // 1.5 ms wake up delay
    }
    else if (!gBrushed_Obj.nSleep && read_nSLEEP_pin)
    {
        set_nSLEEP_lo;                                               // Set nSLEEP LOW
        set_in1_lo;                                                  // Set IN1 to GPIO LOW
        set_in2_lo;                                                  // Set IN2 to GPIO LOW
        gBrushed_Obj.in1 = _PWM_TIMER_PERIOD;                        // Set IN1 to 100 (0% Duty Cycle)
        gBrushed_Obj.in2 = _PWM_TIMER_PERIOD;                        // Set IN2 to 100 (0% Duty Cycle)
        gBrushed_Obj.rampInput1.setpointValue  = _PWM_TIMER_PERIOD;  // Set rampInput1.setpointValue = 100
        gBrushed_Obj.rampInput2.setpointValue  = _PWM_TIMER_PERIOD;  // Set rampInput2.setpointValue = 100
        ignore_ramp = TRUE;
        _delay_cycles(20000);                                        // 1 ms delay for VREF to stabilize
    }

    // Clear Fault using nSLEEP reset pulse
    if (gBrushed_Obj.ClrFlt)
    {
        ClearFaultPulse();                                           
        gBrushed_Obj.ClrFlt = 0;
    }

    if (gBrushed_Obj.deviceID == 8213)
    {
        // Set GAINSEL
        switch (gBrushed_Obj.GAINSEL)
        {
        case 0:
            set_GAINSEL_225;                                  // GAINSEL LOW, 225 µA/A
            break;
        case 1:
            set_GAINSEL_1125;                                 // GAINSEL Hi-Z, 1125 µA/A
            break;
        case 2:
            set_GAINSEL_5560;                                 // GAINSEL LOW, 5560 µA/A
            break;
        default:
            break;
        }

        // IMODE settings
        switch (gBrushed_Obj.IMODE)
        {
        case 0:                                               // No current regulation
            set_IMODE_lo;
            break;
        case 1:                                               // Current regulation at all times (nSTALL=LOW), Current regulation during t_INRUSH only (nSTALL=HIGH)
            set_IMODE_hiz;
            break;
        case 2:                                               // Current regulation at all times
            set_IMODE_hi;
            break;
        default:
            break;
        }

        // SMODE settings
        switch (gBrushed_Obj.SMODE)
        {
        case 0:                                               // Latched
            set_SMODE_lo;
            break;
        case 1:                                               // Indication only: OUTx remain active and nSTALL pulls low
            set_SMODE_hi;
            break;
        case 2:                                               // Indication only: OUTx remains active and nSTALL pulls low. uses internal VREF (500mV) for stall detection
            set_SMODE_hiz;
            break;
        default:
            break;
        }
    }

    // VREF update
    if (gBrushed_Obj.VREF != VREF_previous)
    {
        DAC12_1DAT = 1241*gBrushed_Obj.VREF;
        VREF_previous = gBrushed_Obj.VREF;
        _nop();
    }

    // Stall detection 
    if ((gDeviceI2CReg.FAULT_STATUS & STALL_MASK))
    {
        set_stall_led_high;     // turn on STALL LED indicator
    }
    else
    {
        set_stall_led_low;      // turn off STALL LED indicator
    }

    // Select PWM frequency 200 kHz, 100 kHz, 50 kHz, 25 kHz, 12.5 kHz, 6.25 kHz, 5 kHz, and 3.125 kHz
    if (!gBrushed_Obj.StartDrive1)
    {
        switch (gBrushed_Obj.pwmFreq)
        {
        case 0:
            TB0CTL    |= ID_3 | TBCLR;
            TB0EX0     = TBIDEX__8;                                  // 2.5 kHz
            break;
        case 1:
            TB0CTL    |= ID_3 | TBCLR;
            TB0EX0     = TBIDEX__5;                                  // 5 kHz
            break;
        case 2:
            TB0CTL    |= ID_3 | TBCLR;
            TB0EX0     = TBIDEX__4;                                  // 6.25 kHz
            break;
        case 3:
            TB0CTL    |= ID_3 | TBCLR;
            TB0EX0     = TBIDEX__2;                                  // 12.5 kHz
            break;
        case 4:
            TB0EX0     = TBIDEX__1;
            TB0CTL    |= ID_3 | TBCLR;                               // 25 kHz
            break;
        case 5:
            TB0EX0     = TBIDEX__1;
            TB0CTL    &= ~ID_3;
            TB0CTL    |= ID_2 | TBCLR;                               // 50 kHz
            break;
        case 6:
            TB0EX0     = TBIDEX__1;
            TB0CTL    &= ~ID_3;
            TB0CTL    |= ID_1 | TBCLR;                               // 100 kHz
            break;
        case 7:
            TB0EX0     = TBIDEX__1;
            TB0CTL    &= ~ID_3;
            TB0CTL    |= ID_0 | TBCLR;                               // 200 kHz
            break;
        default:
            break;
        }
    }
}

void UpdateGUIControls() // read device control values and update the GUI values
{
    // Fault Status monitoring
    if (!(read_nFAULT_pin) && gBrushed_Obj.nFault)                  // Fault Condition
    {
        gBrushed_Obj.nFault = 0;                                    // Fault detected
    }
    else if (read_nFAULT_pin)
        gBrushed_Obj.nFault = 1;                                    // No Fault detected

    if (gBrushed_Obj.deviceID == 8213)
    {
        if (!(read_nSTALL_pin) && gBrushed_Obj.nSTALL)                  // Stall Condition
            {
                gBrushed_Obj.nSTALL = 0;                                // Stall detected
            }
        else if (read_nSTALL_pin)
            {
                gBrushed_Obj.nSTALL = 1;                                // No Stall detected
            }
    }
    else
    {
        if (gDeviceI2CReg.FAULT_STATUS & STALL_MASK)
        {
            set_stall_led_high;                                             // Set STALL LED high
            gBrushed_Obj.nSTALL = 0; 
        }
        else 
        {
            set_stall_led_low;
            gBrushed_Obj.nSTALL = 1; 
        }
    } 
}

// Update PWM outputs
void drv_PWM(void)
{
    if (gBrushed_Obj.deviceID == 8213)
    {
        drv_PWM_RampControl();  
    }
    else
    {
        if ((gDeviceI2CReg.CONFIG4 & PMODE_MASK) >> 3) /* PWM mode via PMODE == 1 */
        {
            drv_PWM_RampControl();     
        }
        else
        {
            drv_PH_EN_RampControl();
        }
    }
}

// Function to control the duty cycle for PWM mode 
void drv_PH_EN_RampControl(void)
{
    gBrushed_Obj.dir1 = gBrushed_Obj.in2;

    if (gBrushed_Obj.dir1 && !gBrushed_Obj.pre_dir1)                 // Direction control
    {
        // drv_RampCTRL(&gBrushed_Obj.rampInput1, 0);                   // in1 ramp down before direction reversal
        drv_RampCTRL(&gBrushed_Obj.rampInput1, 0);
        TB0CCR1 = gBrushed_Obj.rampInput1.setpointValue;             // Update Timer PWM duty cycle

        if (!gBrushed_Obj.rampInput1.setpointValue)
        {
            gBrushed_Obj.pre_dir1 = 1;
            P4OUT |= IN2_PIN;                                        // forward = PH/IN2 == 1
        }
    }
    else if (!gBrushed_Obj.dir1 && gBrushed_Obj.pre_dir1)            // Direction control
    {
        // drv_RampCTRL(&gBrushed_Obj.rampInput1, 0);                   // in1 ramp down before direction reversal
        drv_RampCTRL(&gBrushed_Obj.rampInput1, 0);
        TB0CCR1 = gBrushed_Obj.rampInput1.setpointValue;             // Update Timer PWM duty cycle

        if (!gBrushed_Obj.rampInput1.setpointValue)
        {
            gBrushed_Obj.pre_dir1 = 0;
            P4OUT &= ~IN2_PIN;                                       // reverse  = PH/IN2 == 0
        }
    }
    else
    {
        // in1 ramp for PH/EN during normal operation
        P4SEL |= IN1_PIN;                                            // Set IN1 to timer PWM output
        drv_RampCTRL(&gBrushed_Obj.rampInput1, gBrushed_Obj.in1);    // PWM ramp up or down
        TB0CCR1 = gBrushed_Obj.rampInput1.setpointValue;             // Update Timer PWM duty cycle
    }

}

// Function to control the duty cycle for PWM mode 
void drv_PWM_RampControl(void)
{
    if (!gBrushed_Obj.in1 && !gBrushed_Obj.rampInput1.setpointValue)
    {   // IN1 = 0 and setpoint is not 0
        P4OUT &= ~IN1_PIN;                                           // set the output to LOW
        P4SEL &= ~IN1_PIN;                                           // Set IN1 to GPIO output
        DRV_RampControl();                                           // equalizer for timing
    }
    else if (gBrushed_Obj.in1 == _PWM_TIMER_PERIOD && gBrushed_Obj.rampInput1.setpointValue == _PWM_TIMER_PERIOD)
    {
        P4OUT |=  IN1_PIN;                                           // set the output to HIGH
        P4SEL &= ~IN1_PIN;                                           // Set IN1 to GPIO output
        DRV_RampControl();                                           // equalizer for timing
    }
    else
    {
        P4SEL |= IN1_PIN;                                            // Set IN1 to timer PWM output
        DRV_RampControl();                                           // ramp up or down the PWM duty cycle
        TB0CCR1 = gBrushed_Obj.rampInput1.setpointValue;             // Set PWM duty cycle
    }

    if (!gBrushed_Obj.in2 && !gBrushed_Obj.rampInput2.setpointValue)
    {
        P4OUT &= ~IN2_PIN;                                            // set the output to LOW
        P4SEL &= ~IN2_PIN;                                           // Set IN2 to GPIO output
        DRV_RampControl();                                           // equalizer for timing
    }
    else if (gBrushed_Obj.in2 == _PWM_TIMER_PERIOD && gBrushed_Obj.rampInput2.setpointValue == _PWM_TIMER_PERIOD)
    {
        P4OUT |= IN2_PIN;                                           // set the output to HIGH
        P4SEL &= ~IN2_PIN;                                           // Set IN2 to GPIO output
        DRV_RampControl();                                           // equalizer for timing
    }
    else
    {
        P4SEL |= IN2_PIN;                                            // Set IN2 to timer PWM output
        DRV_RampControl();                                           // ramp up or down the PWM duty cycle
        TB0CCR3 = gBrushed_Obj.rampInput2.setpointValue;             // Set PWM duty cycle
    }
}

// Function to ramp up or down the PWM duty cycle for in1, in2
void DRV_RampControl(void)
{
    drv_RampCTRL(&gBrushed_Obj.rampInput1, gBrushed_Obj.in1);
    drv_RampCTRL(&gBrushed_Obj.rampInput2, gBrushed_Obj.in2);

} // End function to control the PWM duty cycle 

// End function to ramp up or down the PWM duty cycle for in1, in2
void drv_RampCTRL(volatile RMPCNTL_t* rmpCntl, uint8_t targetValue)
{
    if (rmpCntl->rampDelayMax == 0)
    {
        rmpCntl->setpointValue = targetValue;
    }

    else
    {
        if (rmpCntl->rampDelayCount++ >= rmpCntl->rampDelayMax)
        {
            if (targetValue > rmpCntl->setpointValue)
            {
                rmpCntl->setpointValue++;
            }
            else if (targetValue < rmpCntl->setpointValue)
            {
                rmpCntl->setpointValue--;

            }

            rmpCntl->rampDelayCount = 0;
        }
    }
} // End function to ramp up and down the PWM duty cycle to target duty cycle value

// IPROPIx results calculation
void UpdateipropiReading()
{
    //calculate average of collected samples
    if (SamplesCollected)
    {
        loop = 0;
        ipropi_sum = 0;

        while (loop != NumSamples)
        {
            ipropi_sum += ipropi_Raw[loop];
            loop++;
        }
            ipropi = (ipropi_sum/NumSamples);                            // Divide for average calculation
            SamplesCollected = 0;                                        // Flag to manage 128 samples collection for average calculation
            ADC12CTL0 |= ADC12ENC;                                       // Enable ADC conversions

    }
}

// drv82x2_GUIcurrent display update function
void GUIcurrentdisplay(void)
{
    // GUI current display update
    if (gBrushed_Obj.StartDrive1 || gBrushed_Obj.rampInput1.setpointValue > 0 || gBrushed_Obj.rampInput2.setpointValue > 0)
    {
        gBrushed_Obj.ipropiAdc = ipropi;                                // Update GUI ipropi current display
    }
    else
    {
        gBrushed_Obj.ipropiAdc = 0;
    }

    // End drv82x2 GUI current display update function
}

// Start Motor and Stop Motor function
void StartStopMotor(void)
{
    //Start Motor is pressed, ramp up PWM
    if (gBrushed_Obj.StartDrive1 && !pwmState1)
    {
        pwmState1 = 1;
        ignore_ramp = FALSE;        // duty cycle ramp up enabled
    }
    // End Start Motor is pressed, ramp up PWM

    //Stop Motor is pressed, ramp down PWM
    if (!gBrushed_Obj.StartDrive1 && pwmState1)
    {
        //pwmState1 = 0;
        gBrushed_Obj.in1 = 0;       // set IN1 to 100% duty cycle
        gBrushed_Obj.in2 = 0;       // set IN2 to 100% duty cycle
        ignore_ramp = FALSE;        // duty cycle ramp down enabled
    }
   
    // End Stop Motor is pressed, ramp down PWM
}// End Start Motor and Stop Motor function

// Timer update for PWM ramp up and down and ADC trigger point for IPROPI measurements
void drv_TimerUpdate(void)
{
    if (timerUpdate)
    {
        drv_PWM();
        timerUpdate = 0;

        // Timer update for ADC trigger points, ADC sampling points spread within PWM periods
        ta0ccr1_value += 2;                                          // Increment sampling point in the PWM every cycle

        if (ta0ccr1_value > 98)                                      // Spread ADC trigger points over entire PWM period
            ta0ccr1_value = TA0CCR1_DEFAULT;                         // Restart if value nears 100% duty cycle

        TA0CCR1 = ta0ccr1_value;                                     // Update CCR1 to trigger ADC12 conversion at a different position in the PWM
        // End Timer update for ADC trigger points, ADC sampling points spread within PWM periods
    }
}// End Timer update for PWM ramp up and down and ADC trigger point for IPROPI measurements

// MSP430F5338 MCU initialization
void mcu_init()
{
    WDTCTL = WDTPW | WDTHOLD;                                        // Stop Watchdog Timer on POR and PUC
    clock_Init(); 
    uartInit();
    gpio_Init();
    ta2_init();
    watchdog_Init();
} // End MSP430F5338 MCU initialization

// Clear Fault function for Hardware configuration
void ClearFaultPulse()
{
    clear_fault_loop = 0;
    while (clear_fault_loop < 1)
    {
        // clear fault procedure for DRV8214/DRV8234 differ from DRV8213
        //DRV8214/34: clear fault by toggling nSLEEP LOW to HIGH with a 30µs wait time
        if (gBrushed_Obj.deviceID == 8214 || gBrushed_Obj.deviceID == 8234 || gBrushed_Obj.deviceID == 8215 || gBrushed_Obj.deviceID == 8235 )
        {
            clear_fault_loop ++;
            //gDeviceI2CReg.CONFIG4 |= Clr_Fault;                // Set Clr_Fault High
            gDeviceI2CReg.CONFIG0 |= 0x02;
            i2c_write((uint8_t) I2C_CONFIG0, (uint8_t) gDeviceI2CReg.CONFIG0);
            _delay_cycles(600);                                // ~30us @ 20 MHz MCLK
        }
        // DRV8213: clear fault by toggling IN1 and IN2 LOW to HIGH with a 700µs delay (stall retry time)
        else
        {
            clear_fault_loop ++;
            set_in1_lo;                                         // set IN1 as GPIO LOW
            set_in2_lo;                                         // set IN2 as GPIO LOW
            _delay_cycles(30000);                               // 700µs retry time
            set_in1_hi;                                         // set IN1 as GPIO HIGH
            set_in2_hi;                                         // set IN2 as GPIO HIGH
        }
    }
}

//This function synchronizes the device registers with the GUI
void gui_regUpdateTask(void)
{
    volatile uint8_t drvRegAddr;
    volatile uint8_t drvDataNew;

    // Write to single register using manual register write command
    if (gBrushed_Obj.ManWriteCmd)
    {
        gBrushed_Obj.ManWriteCmd = FALSE;
        drvRegAddr = gBrushed_Obj.ManWriteAddr;
        drvDataNew = gBrushed_Obj.ManWriteData;

        // I2C write function
        i2c_write((uint8_t) drvRegAddr, (uint8_t) drvDataNew);

        // This function will update the register global variable for the specific register that is manually written
        switch (drvRegAddr)
        {
        case I2C_FAULT_STATUS:
            gDeviceI2CReg.FAULT_STATUS = drvDataNew;
            break;
        case I2C_RC_STATUS_1:
            gDeviceI2CReg.RC_STATUS_1 = drvDataNew;
            break;
        case I2C_RC_STATUS_2:
            gDeviceI2CReg.RC_STATUS_2 = drvDataNew;
            break;
        case I2C_RC_STATUS_3:
            gDeviceI2CReg.RC_STATUS_3 = drvDataNew;
            break;
        case I2C_REG_STATUS_1:
            gDeviceI2CReg.REG_STATUS_1 = drvDataNew;
            break;
        case I2C_REG_STATUS_2:
            gDeviceI2CReg.REG_STATUS_2 = drvDataNew;
            break;
        case I2C_REG_STATUS_3:
            gDeviceI2CReg.REG_STATUS_3 = drvDataNew;
            break;
        case I2C_CONFIG0:
            gDeviceI2CReg.CONFIG0 = drvDataNew;
            break;
        case I2C_CONFIG1:
            gDeviceI2CReg.CONFIG1 = drvDataNew;
            break;
        case I2C_CONFIG2:
            gDeviceI2CReg.CONFIG2 = drvDataNew;
            break;
        case I2C_CONFIG3:
            gDeviceI2CReg.CONFIG3 = drvDataNew;
            break;
        case I2C_CONFIG4:
            gDeviceI2CReg.CONFIG4 = drvDataNew;
            break;
        case I2C_REG_CTRL0:
            gDeviceI2CReg.REG_CTRL0 = drvDataNew;
            break;
        case I2C_REG_CTRL1:
            gDeviceI2CReg.REG_CTRL1 = drvDataNew;
            break;
        case I2C_REG_CTRL2:
            gDeviceI2CReg.REG_CTRL2 = drvDataNew;
            break;
        case I2C_RC_CTRL0:
            gDeviceI2CReg.RC_CTRL0 = drvDataNew;
            break;
        case I2C_RC_CTRL1:
            gDeviceI2CReg.RC_CTRL1 = drvDataNew;
            break;
        case I2C_RC_CTRL2:
            gDeviceI2CReg.RC_CTRL2 = drvDataNew;
            break;
        case I2C_RC_CTRL3:
            gDeviceI2CReg.RC_CTRL3 = drvDataNew;
            break;
        case I2C_RC_CTRL4:
            gDeviceI2CReg.RC_CTRL4 = drvDataNew;
            break;
        case I2C_RC_CTRL5:
            gDeviceI2CReg.RC_CTRL5 = drvDataNew;
            break;
        case I2C_RC_CTRL6:
            gDeviceI2CReg.RC_CTRL6 = drvDataNew;
            break;                                                                                                                                                                                                                        
        default:
            break;
        }
    }

    // Manual Read command
    if (gBrushed_Obj.ManReadCmd)
    {
        gBrushed_Obj.ManReadCmd = FALSE;
        drvRegAddr = gBrushed_Obj.ManReadAddr;
        drvDataNew = i2c_read((uint8_t)drvRegAddr);

        // This function will update the register global variable for the specific register that is manually read
        switch (drvRegAddr)
        {
        case I2C_FAULT_STATUS:
            gDeviceI2CReg.FAULT_STATUS = drvDataNew;
            break;
        case I2C_RC_STATUS_1:
            gDeviceI2CReg.RC_STATUS_1 = drvDataNew;
            break;
        case I2C_RC_STATUS_2:
            gDeviceI2CReg.RC_STATUS_2 = drvDataNew;
            break;
        case I2C_RC_STATUS_3:
            gDeviceI2CReg.RC_STATUS_3 = drvDataNew;
            break;
        case I2C_REG_STATUS_1:
            gDeviceI2CReg.REG_STATUS_1 = drvDataNew;
            break;
        case I2C_REG_STATUS_2:
            gDeviceI2CReg.REG_STATUS_2 = drvDataNew;
            break;
        case I2C_REG_STATUS_3:
            gDeviceI2CReg.REG_STATUS_3 = drvDataNew;
            break;
        case I2C_CONFIG0:
            gDeviceI2CReg.CONFIG0 = drvDataNew;
            break;
        case I2C_CONFIG1:
            gDeviceI2CReg.CONFIG1 = drvDataNew;
            break;
        case I2C_CONFIG2:
            gDeviceI2CReg.CONFIG2 = drvDataNew;
            break;
        case I2C_CONFIG3:
            gDeviceI2CReg.CONFIG3 = drvDataNew;
            break;
        case I2C_CONFIG4:
            gDeviceI2CReg.CONFIG4 = drvDataNew;
            break;
        case I2C_REG_CTRL0:
            gDeviceI2CReg.REG_CTRL0 = drvDataNew;
            break;
        case I2C_REG_CTRL1:
            gDeviceI2CReg.REG_CTRL1 = drvDataNew;
            break;
        case I2C_REG_CTRL2:
            gDeviceI2CReg.REG_CTRL2 = drvDataNew;
            break;
        case I2C_RC_CTRL0:
            gDeviceI2CReg.RC_CTRL0 = drvDataNew;
            break;
        case I2C_RC_CTRL1:
            gDeviceI2CReg.RC_CTRL1 = drvDataNew;
            break;
        case I2C_RC_CTRL2:
            gDeviceI2CReg.RC_CTRL2 = drvDataNew;
            break;
        case I2C_RC_CTRL3:
            gDeviceI2CReg.RC_CTRL3 = drvDataNew;
            break;
        case I2C_RC_CTRL4:
            gDeviceI2CReg.RC_CTRL4 = drvDataNew;
            break;
        case I2C_RC_CTRL5:
            gDeviceI2CReg.RC_CTRL5 = drvDataNew;
            break;
        case I2C_RC_CTRL6:
            gDeviceI2CReg.RC_CTRL6 = drvDataNew;
            break;
        default:
            break;
        }
    }

    // Read all registers at once
    if (ReadRegs && gBrushed_Obj.nSleep)
    {
        ReadAllRegs();
        ReadRegs = FALSE;
    }
}

// Takes the RC_CNT register and converts it to analog output from DAC0
void rc_out_dac_cal(void)
{
    // Perform DAC conversion only if RC value has change from previous value
    //if(RC_CNT_REG != RC_previous)
    //{
        RC_CNT_REG1 = gDeviceI2CReg.RC_STATUS_1;
        RC_CNT_REG2 = gDeviceI2CReg.RC_STATUS_2;

        RC_CNT_REG = ((RC_CNT_REG1 & 0xFF) << 8) | (RC_CNT_REG2 & 0xF0);

        // Write data to output buffer and set current value equals to the current value
        DAC12_0DAT = RC_SCALE*RC_CNT_REG;
        RC_previous = RC_CNT_REG;
        _nop();
    //}
}

void ReadAllRegs(void)
{
    gDeviceI2CReg.FAULT_STATUS  = i2c_read((uint8_t)I2C_FAULT_STATUS);
    gDeviceI2CReg.RC_STATUS_1   = i2c_read((uint8_t)I2C_RC_STATUS_1 );
    gDeviceI2CReg.RC_STATUS_2   = i2c_read((uint8_t)I2C_RC_STATUS_2 );
    gDeviceI2CReg.RC_STATUS_3   = i2c_read((uint8_t)I2C_RC_STATUS_3 );
    gDeviceI2CReg.REG_STATUS_1  = i2c_read((uint8_t)I2C_REG_STATUS_1);
    gDeviceI2CReg.REG_STATUS_2  = i2c_read((uint8_t)I2C_REG_STATUS_2);
    gDeviceI2CReg.REG_STATUS_3  = i2c_read((uint8_t)I2C_REG_STATUS_3);
    gDeviceI2CReg.CONFIG0       = i2c_read((uint8_t)I2C_CONFIG0     );
    gDeviceI2CReg.CONFIG1       = i2c_read((uint8_t)I2C_CONFIG1     );
    gDeviceI2CReg.CONFIG2       = i2c_read((uint8_t)I2C_CONFIG2     );
    gDeviceI2CReg.CONFIG3       = i2c_read((uint8_t)I2C_CONFIG3     );
    gDeviceI2CReg.CONFIG4       = i2c_read((uint8_t)I2C_CONFIG4     );
    gDeviceI2CReg.REG_CTRL0     = i2c_read((uint8_t)I2C_REG_CTRL0   );
    gDeviceI2CReg.REG_CTRL1     = i2c_read((uint8_t)I2C_REG_CTRL1   );
    gDeviceI2CReg.REG_CTRL2     = i2c_read((uint8_t)I2C_REG_CTRL2   );
    gDeviceI2CReg.RC_CTRL0      = i2c_read((uint8_t)I2C_RC_CTRL0    );
    gDeviceI2CReg.RC_CTRL1      = i2c_read((uint8_t)I2C_RC_CTRL1    );
    gDeviceI2CReg.RC_CTRL2      = i2c_read((uint8_t)I2C_RC_CTRL2    );
    gDeviceI2CReg.RC_CTRL3      = i2c_read((uint8_t)I2C_RC_CTRL3    );
    gDeviceI2CReg.RC_CTRL4      = i2c_read((uint8_t)I2C_RC_CTRL4    );
    gDeviceI2CReg.RC_CTRL5      = i2c_read((uint8_t)I2C_RC_CTRL5    );
    gDeviceI2CReg.RC_CTRL6      = i2c_read((uint8_t)I2C_RC_CTRL6    );
}

void RC_CNT_plot(void)
{
    
}

void ReverseDirection(void)
{
    // I2C_BC == 1, bridge control via I2C_EN_IN1 bit and I2C_PH_IN2 bit
    // and PH/EN mode (via PMODE==0)
    if((gDeviceI2CReg.CONFIG4 & I2CBC_MASK) && !(gDeviceI2CReg.CONFIG4 & PMODE_MASK))
    {
        if (gDeviceI2CReg.CONFIG4 & IN2_MASK)
        {
            gDeviceI2CReg.CONFIG4 &= ~IN2_MASK;
            i2c_write(I2C_CONFIG4, gDeviceI2CReg.CONFIG4);
        }
        else
        {
            gDeviceI2CReg.CONFIG4 |= IN2_MASK;
            i2c_write(I2C_CONFIG4, gDeviceI2CReg.CONFIG4);
        }
    }
    // I2C_BC == 1, bridge control via I2C_EN_IN1 bit and I2C_PH_IN2 bit
    // and PWM mode (via PMODE==1)
    else if((gDeviceI2CReg.CONFIG4 & I2CBC_MASK) && (gDeviceI2CReg.CONFIG4 & PMODE_MASK))
    {
        gDeviceI2CReg.CONFIG4 ^= IN1_MASK;
        gDeviceI2CReg.CONFIG4 ^= IN2_MASK;
        i2c_write(I2C_CONFIG4, gDeviceI2CReg.CONFIG4);
    }
    else
    {
        // Really we should use the PWM ramp rate for this in my opinion but okay...
        uint8_t in1_temp = 0;
        uint8_t in2_temp = 0;

        in1_temp = gBrushed_Obj.in1;
        in2_temp = gBrushed_Obj.in2;
        gBrushed_Obj.in1 = 0;
        gBrushed_Obj.in2 = 0;
        __delay_cycles(2000000);
        gBrushed_Obj.in1 = in2_temp;
        gBrushed_Obj.in2 = in1_temp;

        drv_PWM();
    }
}

// Function to reverse motor direction when Ripple Count (RC) reaches the RC Threshold
// Function only works when I2C_BC=0b (IN1 and IN2 controlled via external pins)
void RC_CNT_DIR(void)
{
    uint8_t CNT_DONE = i2c_read((uint8_t) I2C_FAULT_STATUS) & RC_CNT_MASK;
    // Reverse direction only when DIR==1 (Reverse Direction widget) and Ripple Counting tab is selected in the GUI
    // Only reverse direction when CNT_DONE=1b (RC threshold is met) and RC_HIZ=0b (H-bridge is NOT disable when CNT_DONE=1b)
    if(CNT_DONE_temp != CNT_DONE && gBrushed_Obj.nSleep && gBrushed_Obj.dir)
    {
        gDeviceI2CReg.CONFIG0 |= CLR_CNT_MASK;
        i2c_write(I2C_CONFIG0, gDeviceI2CReg.CONFIG0);
        ReverseDirection(); 
        CNT_DONE_temp = 0;
    }
}
