/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *  main.c - c file for main function
 *  DRV8213_82x4EVM_FIRMWARE
 *  03/15/2023
 *  Pablo Armet
 ******************************************************************************/
#include <msp430.h>
#include <stdint.h>
#include "drv82xx/include/drv82xx.h"
#include "Serial_Cmd_Monitor.h"
#include "I2C.h"

volatile unsigned char Adcresults = 0;
volatile unsigned char SamplesCollected = 0;
unsigned int ipropi_Raw[NumSamples];

// Main
int main(void)
{

    mcu_init(); // Initialize MCU peripherals
    getEVMID(); // Read EVM ID, REV ID and set value for deviceID and revID global variables
    gpio_update();
    brushed_init();   // GUI brushed variables configurations
    pwmTimerConfig(); // Timer PWM configuration and initialization
    adcTimerConfig(); // Timer A0 Initialization for ADC12_A internal Timer trigger ADC12SHSx = 1
    adc12_Init();     // Configure ADC channels for IPROPI measurement.

    if (gBrushed_Obj.deviceID == 8214 || gBrushed_Obj.deviceID == 8234 || gBrushed_Obj.deviceID == 8215 || gBrushed_Obj.deviceID == 8235)
    {
        i2c_init(); // i2c init
    }

    // Serial protocol between GUI USB and MCU UART initialization
    ClearBufferRelatedParam(); // GUI Composer Monitor Initialization
    __bis_SR_register(GIE);    // Global interrupts enable

    while (1)
    {
        WDTCTL = WDT_ARST_250; // Watchdog mode 250 ms with ACLK

        // Full-Bridge Brushed DC motor driver variants
        UpdateDeviceControls(); // read the GUI values and update device control values
        if (gBrushed_Obj.deviceID == 8214 || gBrushed_Obj.deviceID == 8234 || gBrushed_Obj.deviceID == 8215 || gBrushed_Obj.deviceID == 8235)
        {
            gui_regUpdateTask(); // Read/Write to DRV8214/34 registers and update GUI register map
            rc_out_dac_cal();    // RC_CNT DAC conversion function
            RC_CNT_DIR();
        }
        UpdateGUIControls(); // read device control values and update the GUI values
        if (!ignore_ramp)
        {
            drv_PWM(); // Stop drv_PWM function when motor start drive is not pressed
        }
        UpdateipropiReading(); // IPROPIx results calculation
        StartStopMotor();      // Start Motor and Stop Motor function
        GUIcurrentdisplay();   // drv82x2 GUI current display update function
        drv_TimerUpdate();     // Timer update for PWM ramp up and down and ADC trigger point for IPROPI measurements

        __bis_SR_register(LPM0_bits + GIE); // Enter LPM0
    }
}

// Timer2_A0 Interrupt Service Routine, periodic wake up and Status LED timing derived from timer
#pragma vector=TIMER2_A0_VECTOR
__interrupt void TimerA2CCR0(void)
{
    REG_UPDATE++;
    LED_TOGGLE_CNT++;
    DIR_CHNG_UPDATE++;
    __bic_SR_register_on_exit(LPM0_bits);             // Clear LPM0 bits
}// End Timer2_A0 Interrupt Service Routine, periodic wake up and Status LED timing derived from timer

#pragma vector=TIMER0_B0_VECTOR
__interrupt void TimerB0(void)
{
    timerUpdate = 1;
}

// ADC12_A Interrupt Service Routine to read drv82xx IPROPIx outputs
#pragma vector=ADC12_VECTOR
__interrupt void ADC12_ISR(void)
{

    ipropi_Raw[Adcresults] = ADC12MEM1 * 1.5;         // IPROPI1 result stored to the array from ADC12MEM1
    Adcresults++;                                     // Advance array pointer

    if (Adcresults == NumSamples)                     // All samples collected?
    {
        ADC12CTL0 &= ~ADC12ENC;                       // Disable ADC until average calculation function is executed
        SamplesCollected = 1;                         // Flag to manage 128 samples collection for average calculation
        Adcresults = 0;                               // Reset array pointer
    }
}// End ADC12_A Interrupt Service Routine to read drv82x2 and drv89x2 IPROPIx outputs
