/* --COPYRIGHT--,BSD
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*******************************************************************************
 *
 *  uart.c - c file for UART communication via UART to USB to the PC GUI app
 *
 ******************************************************************************/
#include "drv82xx/include/drv82xx.h"
#include "uart.h"

unsigned int txData;
extern void receivedDataCommand(unsigned char receiveData);

// This function initializes configuration for UCA1 USCI in UART communication mode
void uartInit(void)
{
	P8SEL = UART_RXD | UART_TXD;                 // P8.3 = UART_RXD, P8.2 = UART_TXD
	UCA1CTL1 |= UCSSEL_2;                        // SMCLK

	// 9600 Baud @ 20 MHz SMCLK
    UCA1BR0 = 0x23;
    UCA1BR1 = 0x08;
    UCA1MCTL = 0x04;

    /*// 115.2k Baud @ 20 MHz SMCLK
	UCA1BR0 = 0xAD;
	UCA1BR1 = 0x0;
	UCA1MCTL = 0x0A;*/

	// Initialize USCI state machine
    UCA1CTL1 &= ~UCSWRST;

	// Enable USCI_A1 RX interrupt
    UCA1IE |= UCRXIE;
}

// This function transmits a character via UART
void uartTxByte(unsigned char byte)
{
	while (!(UCA1IFG & UCTXIFG));                // USCI_A1 TX buffer ready?
	UCA1TXBUF = byte;                            // Transmit data
}

// UART Receive interrupt service routine to receive the MDBU data packet
#pragma vector=USCI_A1_VECTOR
__interrupt void USCI0RX_ISR(void)
{
    receivedDataCommand(UCA1RXBUF);              // Parse received command immediately
    P3OUT ^= GUI_STATUS;                         // Toggle GUI_STATUS LED
} // end USCI0RX_ISR()
